; LnxSpectrum ASM editor (www.ilnx.cz) 17.03.2023
;disk operations- save+ load
;Esxdos
;Hood
;v1.0

;/*
;usage:
;LOAD: uses these calls in the listed order:
;1) file_exist...	checks, if file in "exp_name" exists on the disk, NZ= does not exist
;2) load_init....	prepare given file. NC= file is smaller than buffer
;3) load_file....	NC=we still have portion of file to load

;In the course of loading:
;rest_len........	length of file in bytes, diminished by disk_buf_len each load save. Starts with blen
;portion_len.....	either whole buffer length in bytes, or last part of the file< buffer in bytes


;SAVE: use these calls in listed order:
;1) save_file_head	open file for saving
;2) save_into_file	save file
;3) save_close....	close the file
;*/

	org	32768

	call	fill

dos_init	call	GetSetDrive

fopn	call	fOpen
	call	fSeek
fwrite	call	fWritee
fclose	jp	fClosee

fill	ld	a,"1"
	ld	hl,40000
	ld	de,40001
	ld	bc,1024
	ld	(hl),a
	push	bc
	ldir

;	ld	a,"2"
	inc	a
	pop	bc
	push	bc
	ld	(hl),a
	ldir
	
;	ld	a,"3"
	inc	a
	pop	bc
	ld	(hl),a
	ldir
	ret




;len_buf	equ buf_len		;=#8000	;for MB03+, buffer len has to be always *#400



;temporary
exp_name	db "chemich",0

fnum		dw #5555		;file number to be saved/loaded
beg_sec		dw #5555		;beginning sector of file (vary when saving/loading)
sec_num		db #55			;number of sectors to save/load
flen_secs	dw #5555		;length of file in sectors
disk_buf_len	dw #5555		;length of save/load buffer in bytes
buf_len_secs	db #55			;length of save/load buffer in sectors
portion_len	dw #5555		;disk_buf_len, or rest of file (e.g. #8000, #8000, #03f5)
rest_len	dw #5555,#5555		;length of file in bytes, diminished by disk_buf_len each load save. Starts with blen
tmp_file_sec	dw #5555		;auxiliary. Temporary length of file in sectors. Serves for next_params routine

header
ident		db #B0			;file identification	;#80 when create, #B0 when save done
		ds 4			;time& date		;varaible
ftyp		db #55			;tape header		;variable
fname		ds 10			;file name		;variable
flen		dw #5555		;file length for tape	;variable
fsta		dw #5555
		dw #5555
		dw #5555		;body address
blen		ds 4			;body length
		db #FF			;flagbyte		;always FF when save
		db 0			;attrs			;always 0 when save
		dw 0			;1st sec		;always 0 when save header



Path
file_handle	defb	#55		;Esxdos file handle

;Poznámky od Lanexe 12:37 06.02.2023
;
;U fseek: vynechej první 3 instrukce (test na otevření souboru), nejsou nutné, u fread taky nejsou :)
;Po každém api volání testuj Carry na chybu, a CloseFile použij vždy, když k nějaké chybě během čtení/zápisu dojde.
;
;
; NOTE: File paths use the slash character (‘/’) as directory separator (UNIX style)
;

M_GETSETDRV  equ $89
F_OPEN       equ $9a
F_CLOSE      equ $9b
F_READ       equ $9d
F_WRITE      equ $9e
F_SEEK       equ $9f
F_GETPOS     equ #A0
F_GET_DIR    equ $a8
F_SET_DIR    equ $a9

FA_READ      equ $01
FA_WRITE     equ $02
FA_CREATE_NEW equ $04
FA_APPEND    equ $0A
FA_OVERWRITE equ $0C

; Function: Detect if unit is ready
; Out:      A = default drive (required for all file access)
;           Carry flag will be set if error.
GetSetDrive:
             xor  a               ; A=0, get the default drive
             rst  $08
             db   M_GETSETDRV             
             ld   (DefaultDrive),a
             ret
DefaultDrive db   0



; Function:  Open file
; In:        IX = filename
;            B  = open mode
;            A  = Drive
; Out:       A  = file handle
;            On error: Carry set
;                      A = 5   File not found
;                      A = 7   Name error - not 8.3?
;                      A = 11  Drive not found
;                      


save_file_head
fOpen:
		ld de,0		;find out len of file in secs
		ld hl,1024*3
		ld (rest_len),hl
		ld (rest_len+2),de

;		jp	esx_count_save_len

             ld   a, (DefaultDrive)  ; get drive we're on
	ld	b,FA_APPEND  ;open+append file	
;             ld   b,FA_OVERWRITE
;             ld   b,FA_CREATE_NEW
;             ld   b, FA_READ         ; b = open mode
             ld   ix,exp_name        ; ix = Pointer to file name (ASCIIZ)
             rst  $08
             db   F_OPEN             ; open read mode
             ld	  (file_handle),a  
             ret                     ; Returns a file handler in 'A' register.
;		jr	nc,sfh1		
		jr	nc,esx_count_save_len



disk_error	call	fClose			;carry=error
		jr	d_err

; Function:  Write bytes to a file
; In:        A  = file handle
; Out:       Carry flag active if error when closing
save_close
fClosee: 
             ld   a,(file_handle)           ; a  = file handler
             rst  $08
             db   F_CLOSE
	ret	


d_err		ld c,15
		halt
;		jp error_handler	;NC=disk error, return to main program


	


; Function:  Read bytes from a file
; In:        A  = file handle
;            ix = address to load into
;            bc = number of bytes to read
; Out:       Carry flag is set if read fails.
fRead:
             ld   ix, 16384          ; ix = address where to store what is read
             ld   bc, 6912           ; bc = bytes to read
             ld   a,(file_handle)      ; a  = the file handler
             rst  $08
             db   F_READ             ; read file
             ret

save_into_file
; Function:  Write bytes to a file
; In:        A  = file handle
;            ix = address to save from 
;            bc = number of bytes to write
; Out:       Carry flag is set if write fails.
fWritee:
;		halt
;		ret

sif1         ld   ix, 40000      ; ix = memory address to save from
sif2         ld   bc,1024           ; bc = bytes to write
             ld   a,(file_handle)    ; a  = file handler
             rst  $08
             db   F_WRITE            ; write file
             ret
		jr	c,disk_error
;sf1		jp next_params
;		ret
esx_count_save_len			;count if save will be whole buffer or less (in case of last part)
		ld	hl,(disk_buf_len)
		ld	bc,(rest_len+2)
		ld	a,b
		or	c
		jr	nz,ecsl1	;NZ=save the whole buffer

		ld	bc,(rest_len)
		or	a
		sbc	hl,bc
		add	hl,bc
		jr	c,ecsl1
		ld	h,b		;save last portion (=buffer length or less)
		ld	l,c

ecsl1		ld	(sif2+1),hl
		ret



; Function:  Seek into file
; In:        A    = file handle
;            L    = mode:  0 - from start of file
;                          1 - forward from current position
;                          2 - back from current position
;            BCDE = bytes to seek
; Out:       BCDE = Current file pointer. (*does not return this yet)
;            
fSeek:   
             ld   a, (file_handle)   ; file handle
             or   a                  ; is it zero?
             ret  z                  ; if so return
             ld   ixl,0
             ld   bc,#FFFF
             ld   de,#FFFF
             rst  $08           
	db F_GETPOS
 ;            db   F_SEEK
             ret



; Function:  SetDirectory
; In:        A    = Drive
;            HL   = pointer to zero terminated path string ("path",0)
; Out:       carry set if error
;            
SetDir:
             ld   a,(DefaultDrive)   ; drive to change directory on
             ld   hl,Path            ; point to "path",0 to set
             rst  $08           
             db   F_SET_DIR  
             ret



; Function:  Get Directory
; In:        A    = Drive
;            HL   = pointer to where to STORE zero terminated path string
; Out:       carry set if error
;            
GetDir:     
             ld   a,(DefaultDrive)   ; drive to get current directory from
             ld   hl,Path            ; location to store path string in
             rst  $08           
             db   F_GET_DIR  
             ret




delete_file	ret

;/*
;delete_file	ld bc,(fnum)
		ld a,erasex
		call bsdos
		jr nc,disk_error
		ret


;auxiliary routines


;zkusime vylepsit
next_params	ld	hl,(rest_len)	;substract disk_buf_len from rest_file first
		ld	de,(rest_len+2)	;rest_len can be then used eg. for display copy progress
		ld	bc,(disk_buf_len)
		ld	a,sub32
		call	bsdos
		ld	(rest_len+2),de
		ld	(rest_len),hl

next_params1	ld	hl,(beg_sec)
		ld	a,(buf_len_secs)
		ld	d,0
		ld	e,a
		add	hl,de
		ld	(beg_sec),hl	;inccrease starting sector by buffer length

		ld	hl,(tmp_file_sec)
		or	a
		sbc	hl,de		;substract buffer length from temp file length
		ld	(tmp_file_sec),hl
		ld	a,h
		or	a
		jr	nz,np_next	;NZ=NC= still portion to load/save
		ld	a,l
		cp	e
		jr	nc,np_next	
		ld	e,l
np_next		ld	a,e
		ld	(sec_num),a
		ld	b,a

		push	af
		ld	hl,(flen_secs)	;if file is < buffer, we have to deal it
		ld	a,h
		or	a
		jr	nz,npn1		;NZ=file> buffer
		ld	a,(buf_len_secs)
		cp	l		;L=length of file in secs
		ld	hl,(blen)
		jr	nc,npn2
npn1		ld	hl,(disk_buf_len);here we continue. Count rest of length, if needed
npn2		ld	(portion_len),hl
		pop	af
		ret	nc		;NC-we still have a portion to load/save, it is full buffer, return
		ld 	hl,last_portion	;if last part to load, modify the jump after load last part
;		ld 	(lf1+1),hl
		ret


last_portion	ld	hl,blen		;ziskej delku posledni casti v bajtech na zaklade poctu poslednich sektoru.
					;Pokud je spodnich 10 bitu delky souboru 0, pak je posledni sektor cely obsazeny.
					;Pak staci pocet sektoru rotovat o 10 bitu vlevo a mam vyslednou delku.
					;Jinak rotuj o 10 bitu vlevo pocet sektoru zmenseny o 1 a pripocti k tomu 10 bitu celkove 						;delky souboru
		ld	a,(sec_num)
		ld	b,a
		ld	a,(hl)
		or	a
		jr	nz,nx_end2
		inc	hl
		ld	a,(hl)
		and	3
		jr	nz,nx_end2

		sla	b		;posledni sektor je cely obsazeny, rotuj pocet sektoru o 10 bitu vlevo
		sla	b
		ld	h,b
		ld	l,0
;		ld	c,0		;chyba
		jr	nx_end1		

nx_end2		dec	b		;A=pocet sektoru, posledni sektor je castecne obsazeny
		sla	b
		sla	b
		ld 	hl,blen+1
		ld	a,(hl)
		and	3
		or	b
		ld	b,a
		dec	hl
		ld	l,(hl)
		ld	h,b
nx_end1		ld	(portion_len),hl	;HL= delka v bajtech zbyvajici casti. Bud cely buffer, nebo zbytek souboru
		scf
		ret			



	
sub32		equ #B
howsec		equ #F
getcst		equ #12
getsub		equ #16
seasx		equ #19
owsub		equ #1B
loa1x		equ #22
savex		equ #25
creatx		equ #26
erasex		equ #28


bsdos		xor	a		;6 bajtů
		sbc	hl,bc
		ret	nc
		dec	de
		ret
;	ENDIF

end
;len=end-start_address
